var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
* Copyright 2018 ARDUINO SA (http://www.arduino.cc/)
* This file is part of arduino-create-agent-js-client.
* Copyright (c) 2018
* Authors: Alberto Iannaccone, Stefania Mellai, Gabriele Destefanis
*
* This software is released under:
* The GNU General Public License, which covers the main part of
* arduino-create-agent-js-client
* The terms of this license can be found at:
* https://www.gnu.org/licenses/gpl-3.0.en.html
*
* You can be released from the requirements of the above licenses by purchasing
* a commercial license. Buying such a license is mandatory if you want to modify or
* otherwise use the software for commercial activities involving the Arduino
* software without disclosing the source code of your own applications. To purchase
* a commercial license, send an email to license@arduino.cc.
*
*/

import io from 'socket.io-client';
import semVerCompare from 'semver-compare';
import { detect } from 'detect-browser';

import { timer } from 'rxjs';
import { filter, takeUntil, first } from 'rxjs/operators';

import Daemon from './daemon';

// Required agent version
var browser = detect();
var POLLING_INTERVAL = 3500;
var UPLOAD_DONE_TIMER = 5000;

var PROTOCOL = {
  HTTP: 'http',
  HTTPS: 'https'
};

var LOOPBACK_ADDRESS = PROTOCOL.HTTP + '://127.0.0.1';
var LOOPBACK_HOST = PROTOCOL.HTTPS + '://localhost';
var LOOKUP_PORT_START = 8991;
var LOOKUP_PORT_END = 9000;
var orderedPluginAddresses = [LOOPBACK_ADDRESS, LOOPBACK_HOST];
var driversRequested = false;

var CANT_FIND_AGENT_MESSAGE = 'Arduino Create Agent cannot be found';

var updateAttempts = 0;

if (browser.name !== 'chrome' && browser.name !== 'firefox') {
  orderedPluginAddresses = [LOOPBACK_HOST, LOOPBACK_ADDRESS];
}

var SocketDaemon = function (_Daemon) {
  _inherits(SocketDaemon, _Daemon);

  function SocketDaemon(boardsUrl) {
    _classCallCheck(this, SocketDaemon);

    var _this = _possibleConstructorReturn(this, (SocketDaemon.__proto__ || Object.getPrototypeOf(SocketDaemon)).call(this, boardsUrl));

    _this.selectedProtocol = PROTOCOL.HTTP;
    _this.socket = null;
    _this.pluginURL = null;
    _this.disabled = false;

    _this.openChannel(function () {
      return _this.socket.emit('command', 'list');
    });

    _this.agentFound.subscribe(function (agentFound) {
      if (agentFound) {
        _this._wsConnect();
      } else {
        _this.findAgent();
      }
    });
    return _this;
  }

  _createClass(SocketDaemon, [{
    key: 'initSocket',
    value: function initSocket() {
      var _this2 = this;

      this.socket.on('message', function (message) {
        try {
          _this2.appMessages.next(JSON.parse(message));
        } catch (SyntaxError) {
          _this2.appMessages.next(message);
        }
      });
    }
  }, {
    key: 'notifyDownloadError',
    value: function notifyDownloadError(err) {
      this.downloading.next({ status: this.DOWNLOAD_ERROR, err: err });
    }

    /**
     * Look for the agent endpoint.
     * First search in LOOPBACK_ADDRESS, after in LOOPBACK_HOST if in Chrome or Firefox, otherwise vice versa.
     */

  }, {
    key: 'findAgent',
    value: function findAgent() {
      var _this3 = this;

      if (this.disabled) {
        return;
      }
      if (this.pluginURL) {
        fetch(this.pluginURL + '/info').then(function (response) {
          return response.json().then(function (data) {
            _this3.agentInfo = data;
            _this3.agentFound.next(true);
          });
        }).catch(function () {
          return timer(POLLING_INTERVAL).subscribe(function () {
            return _this3.findAgent();
          });
        });
        return;
      }

      this._tryPorts(orderedPluginAddresses[0]).catch(function () {
        return _this3._tryPorts(orderedPluginAddresses[1]);
      }).then(function () {
        return _this3.agentFound.next(true);
      }).catch(function () {
        return timer(POLLING_INTERVAL).subscribe(function () {
          return _this3.findAgent();
        });
      });
    }

    /**
     * Try ports for the selected host. From LOOKUP_PORT_START to LOOKUP_PORT_END
     * @param {string} host - The host value (LOOPBACK_ADDRESS or LOOPBACK_HOST).
     * @return {Promise} info - A promise resolving with the agent info values.
     */

  }, {
    key: '_tryPorts',
    value: function _tryPorts(host) {
      var _this4 = this;

      var pluginLookups = [];

      for (var port = LOOKUP_PORT_START; port < LOOKUP_PORT_END; port += 1) {
        pluginLookups.push(fetch(host + ':' + port + '/info').then(function (response) {
          return response.json().then(function (data) {
            return { response: response, data: data };
          });
        }).catch(function () {
          return Promise.resolve(false);
        }));
        // We expect most of those call to fail, because there's only one agent
        // So we have to resolve them with a false value to let the Promise.all catch all the deferred data
      }

      return Promise.all(pluginLookups).then(function (responses) {
        var found = responses.some(function (r) {
          if (r && r.response && r.response.status === 200) {
            _this4.agentInfo = r.data;

            if (_this4.agentInfo.update_url.indexOf('downloads.arduino.cc') === -1) {
              _this4.error.next('unofficial plugin');
            }

            if (r.response.url.indexOf(PROTOCOL.HTTPS) === 0) {
              _this4.selectedProtocol = PROTOCOL.HTTPS;
            } else {
              // Protocol http, force 127.0.0.1 for old agent versions too
              _this4.agentInfo[_this4.selectedProtocol] = _this4.agentInfo[_this4.selectedProtocol].replace('localhost', '127.0.0.1');
            }
            _this4.pluginURL = _this4.agentInfo[_this4.selectedProtocol];
            return true;
          }
          return false;
        });

        if (found) {
          return fetch('https://s3.amazonaws.com/arduino-create-static/agent-metadata/agent-version.json').then(function (response) {
            return response.json().then(function (data) {
              if (_this4.agentInfo.version && (semVerCompare(_this4.agentInfo.version, data.Version) >= 0 || _this4.agentInfo.version.indexOf('dev') !== -1)) {
                return _this4.agentInfo;
              }

              if (updateAttempts === 0) {
                return _this4.update();
              }
              if (updateAttempts < 3) {
                return timer(10000).subscribe(function () {
                  return _this4.update();
                });
              }
              updateAttempts += 1;
              _this4.error.next('plugin version incompatible');
              return Promise.reject(new Error('plugin version incompatible'));
            });
          })
          // If version API broken, go ahead with current version
          .catch(function () {
            return _this4.agentInfo;
          });
        }

        // Set channelOpen false for the first time
        if (_this4.channelOpen.getValue() === null) {
          _this4.channelOpen.next(false);
        }
        return Promise.reject(new Error(CANT_FIND_AGENT_MESSAGE + ' at ' + host));
      });
    }

    /**
     * Uses the websocket protocol to connect to the agent
     */

  }, {
    key: '_wsConnect',
    value: function _wsConnect() {
      var _this5 = this;

      var wsProtocol = this.selectedProtocol === PROTOCOL.HTTPS ? 'wss' : 'ws';
      var address = this.agentInfo[wsProtocol];

      // Reset
      if (this.socket) {
        this.socket.destroy();
        delete this.socket;
        this.socket = null;
      }

      this.socket = io(address);

      this.socket.on('connect', function () {
        // On connect download windows drivers which are indispensable for detection of boards
        if (!driversRequested) {
          _this5.downloadTool('windows-drivers', 'latest', 'arduino');
          _this5.downloadTool('bossac', '1.7.0', 'arduino');
          driversRequested = false;
        }

        _this5.initSocket();

        _this5.channelOpen.next(true);
      });

      this.socket.on('error', function (error) {
        _this5.socket.disconnect();
        _this5.error.next(error);
      });

      this.socket.on('disconnect', function () {
        _this5.socket.disconnect();
        _this5.channelOpen.next(false);
      });
    }
  }, {
    key: 'handleAppMessage',
    value: function handleAppMessage(message) {
      // Result of a list command
      if (message.Ports) {
        this.handleListMessage(message);
      }
      // Serial monitor message
      if (message.D) {
        this.serialMonitorMessages.next(message.D);
      }

      if (message.ProgrammerStatus) {
        this.handleUploadMessage(message);
      }

      if (message.DownloadStatus) {
        this.handleDownloadMessage(message);
      }

      if (message.Err) {
        this.uploading.next({ status: this.UPLOAD_ERROR, err: message.Err });
      }

      if (message.Error) {
        if (message.Error.indexOf('trying to close') !== -1) {
          // https://github.com/arduino/arduino-create-agent#openclose-ports
          this.serialMonitorOpened.next(false);
        }
      }
    }
  }, {
    key: 'handleListMessage',
    value: function handleListMessage(message) {
      var lastDevices = this.devicesList.getValue();
      if (message.Network && !Daemon.devicesListAreEquals(lastDevices.network, message.Ports)) {
        this.devicesList.next({
          serial: lastDevices.serial,
          network: message.Ports
        });
      } else if (!message.Network && !Daemon.devicesListAreEquals(lastDevices.serial, message.Ports)) {
        this.devicesList.next({
          serial: message.Ports,
          network: lastDevices.network
        });
      }
    }

    /**
     * Check the agent version and call the update if needed.
     */

  }, {
    key: 'update',
    value: function update() {
      var _this6 = this;

      return fetch(this.agentInfo[this.selectedProtocol] + '/update', {
        method: 'POST',
        headers: {
          'Content-Type': 'text/plain; charset=utf-8'
        }
      }).then(function (result) {
        return result.json();
      }).then(function (response) {
        if (!response.ok) {
          if (response && response.error && (response.error.indexOf('proxy') !== -1 || response.error.indexOf('dial tcp') !== -1)) {
            _this6.error.next('proxy error');
            return new Error('proxy error');
          }
        }
        // We reject the promise because the daemon will be restarted, we need to continue looking for the port
        return Promise.reject();
      }).catch(function () {
        console.log('update plugin failed');
      });
    }

    /**
     * Pauses the plugin
     * @return {Promise}
     */

  }, {
    key: 'stopPlugin',
    value: function stopPlugin() {
      if (this.agentFound.getValue()) {
        return fetch(this.agentInfo[this.selectedProtocol] + '/pause', { method: 'POST' });
      }
    }

    /**
     * Send 'close' command to all the available serial ports
     */

  }, {
    key: 'closeAllPorts',
    value: function closeAllPorts() {
      var _this7 = this;

      var devices = this.devicesList.getValue().serial;
      devices.forEach(function (device) {
        _this7.socket.emit('command', 'close ' + device.Name);
      });
    }

    /**
     * Send 'message' to serial port
     * @param {string} port the port name
     * @param {string} message the text to be sent to serial
     */

  }, {
    key: 'writeSerial',
    value: function writeSerial(port, message) {
      this.socket.emit('command', 'send ' + port + ' ' + message);
    }

    /**
     * Request serial port open
     * @param {string} port the port name
     */

  }, {
    key: 'openSerialMonitor',
    value: function openSerialMonitor(port) {
      var _this8 = this;

      var baudrate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 9600;

      var serialPort = this.devicesList.getValue().serial.find(function (p) {
        return p.Name === port;
      });
      if (!serialPort) {
        return this.serialMonitorOpened.error(new Error('Can\'t find board at ' + port));
      }
      if (this.uploading.getValue().status === this.UPLOAD_IN_PROGRESS || serialPort.IsOpen) {
        return;
      }

      this.appMessages.pipe(takeUntil(this.serialMonitorOpened.pipe(filter(function (open) {
        return open;
      })))).subscribe(function (message) {
        if (message.Cmd === 'Open') {
          _this8.serialMonitorOpened.next(true);
        }
        if (message.Cmd === 'OpenFail') {
          _this8.serialMonitorOpened.error(new Error('Failed to open serial monitor at ' + port));
        }
      });

      this.socket.emit('command', 'open ' + port + ' ' + baudrate + ' timed');
    }

    /**
     * Request serial port close
     * @param {string} port the port name
     */

  }, {
    key: 'closeSerialMonitor',
    value: function closeSerialMonitor(port) {
      var _this9 = this;

      var serialPort = this.devicesList.getValue().serial.find(function (p) {
        return p.Name === port;
      });
      if (!serialPort || !serialPort.IsOpen) {
        return;
      }

      this.appMessages.pipe(takeUntil(this.serialMonitorOpened.pipe(filter(function (open) {
        return !open;
      })))).subscribe(function (message) {
        if (message.Cmd === 'Close') {
          _this9.serialMonitorOpened.next(false);
        }
        if (message.Cmd === 'CloseFail') {
          _this9.serialMonitorOpened.error(new Error('Failed to close serial monitor at ' + port));
        }
      });
      this.socket.emit('command', 'close ' + port);
    }
  }, {
    key: 'handleUploadMessage',
    value: function handleUploadMessage(message) {
      var _this10 = this;

      if (message.Flash === 'Ok' && message.ProgrammerStatus === 'Done') {
        // After the upload is completed the port goes down for a while, so we have to wait a few seconds
        return timer(UPLOAD_DONE_TIMER).subscribe(function () {
          return _this10.uploading.next({ status: _this10.UPLOAD_DONE, msg: message.Flash });
        });
      }
      switch (message.ProgrammerStatus) {
        case 'Starting':
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: 'Programming with: ' + message.Cmd });
          break;
        case 'Busy':
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: message.Msg });
          break;
        case 'Error':
          this.uploading.next({ status: this.UPLOAD_ERROR, err: message.Msg });
          break;
        case 'Killed':
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: 'terminated by user' });
          this.uploading.next({ status: this.UPLOAD_ERROR, err: 'terminated by user' });
          break;
        case 'Error 404 Not Found':
          this.uploading.next({ status: this.UPLOAD_ERROR, err: message.Msg });
          break;
        default:
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: message.Msg });
      }
    }
  }, {
    key: 'handleDownloadMessage',
    value: function handleDownloadMessage(message) {
      switch (message.DownloadStatus) {
        case 'Pending':
          this.downloading.next({ status: this.DOWNLOAD_IN_PROGRESS, msg: message.Msg });
          break;
        case 'Success':
          this.downloading.next({ status: this.DOWNLOAD_DONE, msg: message.Msg });
          break;
        case 'Error':
          this.downloading.next({ status: this.DOWNLOAD_ERROR, err: message.Msg });
          break;
        default:
          this.downloading.next({ status: this.DOWNLOAD_IN_PROGRESS, msg: message.Msg });
      }
    }

    /**
     * Perform an upload via http on the daemon
     * @param {Object} data
     */

  }, {
    key: 'daemonUpload',
    value: function daemonUpload(data) {
      var _this11 = this;

      fetch(this.pluginURL + '/upload', {
        method: 'POST',
        headers: {
          'Content-Type': 'text/plain; charset=utf-8'
        },
        body: JSON.stringify(data)
      }).then(function (result) {
        if (!result.ok) {
          _this11.uploading.next({ status: _this11.UPLOAD_ERROR, err: result.statusText });
        }
      }).catch(function (error) {
        _this11.uploading.next({ status: _this11.UPLOAD_ERROR, err: error });
      });
    }

    /**
     * Upload compiled sketch to serial target
     * @param {Object} uploadPayload payload properties defined in parent
     * @param {Object} uploadCommandInfo = {
     *  commandline: "commandline to execute, for serial upload",
        signature: "signature of the commandline",
     *  options: {
     *    wait_for_upload_port: true or false,
     *    use_1200bps_touch: true or false,
     *  },
     *  tools: [{
     *      name: 'avrdude',
     *      packager: 'arduino',
     *      version '6.3.0-arduino9'
     *    },
     *    {...}
     *  ]
     * }
     */

  }, {
    key: '_upload',
    value: function _upload(uploadPayload, uploadCommandInfo) {
      var _this12 = this;

      if (Array.isArray(uploadCommandInfo.tools)) {
        uploadCommandInfo.tools.forEach(function (tool) {
          _this12.downloadTool(tool.name, tool.version, tool.packager);
        });
      }

      var socketParameters = _extends({}, uploadPayload, {
        extra: _extends({}, uploadPayload.extra, {
          wait_for_upload_port: uploadCommandInfo.options.wait_for_upload_port === 'true' || uploadCommandInfo.options.wait_for_upload_port === true,
          use_1200bps_touch: uploadCommandInfo.options.use_1200bps_touch === 'true' || uploadCommandInfo.options.use_1200bps_touch === true
        }),
        extrafiles: uploadCommandInfo.files || []
        // Consider to push extra resource files from sketch in future if feature requested (from data folder)
      });

      if (!socketParameters.extra.network) {
        socketParameters.signature = uploadCommandInfo.signature;
      }

      this.downloadingDone.subscribe(function () {
        _this12.serialMonitorOpened.pipe(filter(function (open) {
          return !open;
        })).pipe(first()).subscribe(function () {
          _this12.daemonUpload(socketParameters);
        });
      });

      this.downloadingError.subscribe(function (error) {
        return _this12.uploading.next({ status: _this12.UPLOAD_ERROR, err: error });
      });
    }

    /**
     * Upload compiled sketch to network target
     * @param {Object} target = {
     *    board: 'fqbn',
     *    port: 'ip address',
     *    extra: {},
     * }
     * @param {string} sketchName
     * @param {Object} compilationResult
     */

  }, {
    key: 'uploadNetwork',
    value: function uploadNetwork(target, sketchName, compilationResult) {
      this.uploading.next({ status: this.UPLOAD_IN_PROGRESS });

      var uploadPayload = _extends({}, target, {
        filename: sketchName + '.hex',
        hex: compilationResult.hex
      });
      this.daemonUpload(uploadPayload);
    }

    /**
     * Upload file to network target (arduino-connector)
     * @param {Object} target
     * @param {string} sketchName
     * @param {Object} encodedFile
     * @param {Object} commandData {commandline: '', signature: ''}
     */

  }, {
    key: 'uploadConnector',
    value: function uploadConnector(target, sketchName, encodedFile, commandData) {
      this.uploading.next({ status: this.UPLOAD_IN_PROGRESS });

      var uploadPayload = _extends({}, target, {
        commandline: commandData.commandline,
        signature: commandData.signature,
        filename: sketchName,
        hex: encodedFile
      });
      this.daemonUpload(uploadPayload);
    }

    /**
     * Download tool
     * @param {string} toolName
     * @param {string} toolVersion
     * @param {string} packageName
     * @param {string} replacementStrategy
     */

  }, {
    key: 'downloadTool',
    value: function downloadTool(toolName, toolVersion, packageName) {
      var replacementStrategy = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'keep';

      this.downloading.next({ status: this.DOWNLOAD_IN_PROGRESS });
      this.socket.emit('command', 'downloadtool ' + toolName + ' ' + toolVersion + ' ' + packageName + ' ' + replacementStrategy);
    }

    /**
     * Interrupt upload
     */

  }, {
    key: 'stopUploadCommand',
    value: function stopUploadCommand() {
      this.uploading.next({
        status: this.UPLOAD_ERROR,
        err: 'upload stopped'
      });
      this.socket.emit('command', 'killupload');
    }
  }, {
    key: 'disable',
    value: function disable() {
      this.disabled = true;
    }
  }, {
    key: 'enable',
    value: function enable() {
      this.disable = false;
      this.findAgent();
    }
  }]);

  return SocketDaemon;
}(Daemon);

export default SocketDaemon;